@@ -18,15 +18,37 @@ Interactive Plots (:mod:`kaleidoscope.interactive`)
 
 .. currentmodule:: kaleidoscope.interactive
 
+Distributions
+=============
+.. autosummary::
+   :toctree: ../stubs/
+
+   probability_distribution
+
+
+Statevectors
+============
 .. autosummary::
    :toctree: ../stubs/
 
-   counts_distribution
    bloch_sphere
    bloch_disc
    bloch_multi_disc
+   qsphere
+
+
+Plotly wrappers
+===============
+
+.. autosummary::
+   :toctree: ../stubs/
+
+   PlotlyFigure
+   PlotlyWidget
 """
 
-from .histogram import counts_distribution
+from .histogram import probability_distribution
 from .bloch.bloch2d import bloch_disc, bloch_multi_disc
 from .bloch.bloch3d import bloch_sphere
+from .qsphere import qsphere
+from .plotly_wrapper import PlotlyFigure, PlotlyWidget
